package sysinfo

import (
	"encoding/json"
	"path/filepath"
	"strings"
)

type NetworkCollector struct{}

func (NetworkCollector) Key() string { return "network" }

func (NetworkCollector) Collect(c *Context) (json.RawMessage, error) {
	def, _ := c.defaultIfaceSet()
	sections := map[string]any{}

	nets, _ := filepath.Glob("/sys/class/net/*")
	for _, n := range nets {
		if strings.HasSuffix(n, "/lo") {
			continue
		}
		iface := filepath.Base(n)
		mac := strings.ToLower(strings.ReplaceAll(
			c.SafeRead(filepath.Join(n, "address")), ":", ""))
		if mac == "" {
			continue
		}
		entry := map[string]any{
			"iface":     iface,
			"timestamp": c.Now.Format(time.RFC3339),
			"link":      c.SafeRun("ip", "-details", "link", "show", iface),
			"ethtool":   c.SafeRun("ethtool", iface),
		}
		ipJSON, _ := c.runCmd("ip", "-j", "address", "show", iface)
		entry["ip_addr"] = json.RawMessage(ipJSON)

		stats, _ := c.readNetStats(iface)
		entry["statistics"] = json.RawMessage(stats)

		if def[iface] {
			if out := c.SafeRun("curl", "--interface", iface, "-s",
				"https://ipinfo.io"); json.Valid([]byte(out)) {
				entry["ipinfo"] = json.RawMessage(out)
			}
		}
		sections[mac] = entry
	}
	return json.Marshal(sections)
}
